# GI case study, normalization and cell clustering using FlowSOM 
# Author: Candace Lu
# Date: 1/21/21

library(FlowSOM)
library(data.table)
library(viridis)
library(pheatmap)
library(RColorBrewer)
library(ggplot2)
set.seed(101)

# Markers for clustering
markers = c("CD45","CD31","SMA","Pan-Keratin","Vimentin","CD3","CD20","CD68","CD209","Mast Cell Chymase","CD56","CD4","CD8","CD14")
# Colors for plotting
all_colors = c(brewer.pal(name="Set3", n=12), brewer.pal(name="Paired", n=12), brewer.pal(name="Pastel1", n=9), brewer.pal(name="Dark2", n=8), brewer.pal(name="Set1", n=9), brewer.pal(name="Pastel2", n=8))
names(all_colors) = paste0("cell_h", c(1:length(all_colors)))


### 1. Cell clustering with FlowSOM  
seg_filename = "seg_cell_transformed_data.csv"
seg_cell_dat = fread(seg_filename)
# Remove rows with all 0
seg_cell_dat = seg_cell_dat[rowSums(seg_cell_dat[,..markers]) != 0]
dat = seg_cell_dat[,..markers]
# Turn into frequency
dat = dat[,lapply(.SD, function(x) x/rowSums(dat))]
# Perform 99.9% normalization
dat = dat[, lapply(.SD, function(x) x/quantile(x,0.999))]

# Cluster data using FlowSOM
fSOM = SOM(data = as.matrix(dat))
dat[,cluster := fSOM$mapping[,1]]
seg_cell_dat[,cluster := fSOM$mapping[,1]]

# Get mean of each cluster
mean_dat = dat[, lapply(.SD, mean), by = cluster]
# Convert expression values back to data frame
mat_dat = data.frame(mean_dat[, ..markers])
rownames(mat_dat) = paste0("clust_", mean_dat$cluster)
# Z-score the columns
mat_dat = scale(mat_dat)
# Determine breaks
breaks = seq(min(mat_dat), max(mat_dat), length.out=100)
# Make heatmap
p = pheatmap(t(mat_dat),
             color = viridis(length(breaks)-1),
             breaks = breaks,
             main = "Average across clusters")
# Get hierarchical clusters
clust_to_hclust = cutree(p$tree_col, k=20)
names(clust_to_hclust) = mean_dat$cluster
mean_dat[,hCluster := clust_to_hclust[as.character(mean_dat$cluster)]]
seg_cell_dat[,hCluster := clust_to_hclust[as.character(seg_cell_dat$cluster)]]
# Annotations
mat_col = data.frame(hclust = paste0("cell_h", mean_dat$hCluster))
rownames(mat_col) = paste0("clust_", mean_dat$cluster)
# Annotation colors
mat_colors = list(hclust = all_colors)
names(mat_colors$hclust) = paste0("cell_h", c(1:length(all_colors)))
# Plot
pdf("seg_cellClusters.pdf", width=20, height=10)
pheatmap(t(mat_dat),
         color = viridis(length(breaks)-1),
         breaks = breaks,
         annotation_col = mat_col,
         annotation_colors = mat_colors,
         annotation_legend = FALSE)
dev.off()

# Cap 0-3
mat_dat = pmin(mat_dat, 3)
mat_dat = pmax(mat_dat, 0)
# Determine breaks
breaks = seq(min(mat_dat), max(mat_dat), length.out=100)
# Make heatmap
p = pheatmap(t(mat_dat),
             color = viridis(length(breaks)-1),
             breaks = breaks,
             main = "Average across clusters")
# Get hierarchical clusters
clust_to_hclust = cutree(p$tree_col, k=20)
names(clust_to_hclust) = mean_dat$cluster
mean_dat[,hCluster_cap := clust_to_hclust[as.character(mean_dat$cluster)]]
seg_cell_dat[,hCluster_cap := clust_to_hclust[as.character(seg_cell_dat$cluster)]]
# Annotations
mat_col = data.frame(hclust = paste0("cell_h", mean_dat$hCluster_cap))
rownames(mat_col) = paste0("clust_", mean_dat$cluster)
# Plot
pdf("seg_cellClusters_cap03.pdf", width=20, height=10)
pheatmap(t(mat_dat),
         color = viridis(length(breaks)-1),
         breaks = breaks,
         annotation_col = mat_col,
         annotation_colors = mat_colors,
         annotation_legend = FALSE)
dev.off()

seg_cell_dat[,point:=gsub("Point","",fov)]
fwrite(seg_cell_dat, "seg_cell_transformed_data_clusters.csv")

# Save clusters only
clust_only = unique(seg_cell_dat[,c("cluster","hCluster","hCluster_cap")])
clust_only = clust_only[order(cluster)]
fwrite(clust_only, "seg_cellClusters_annot.csv")


### 2. Manually annotate 100 clusters from FlowSOM. After manual annotation, add annotations to data table
clusters = fread("seg_cell_transformed_data_clusters.csv")
annotations = fread("seg_cellClusters_annot.csv")

clust_to_pheno = annotations$phenotype
names(clust_to_pheno) = as.character(annotations$cluster)
clusters[,phenotype:=clust_to_pheno[as.character(cluster)]]

clust_to_lineage = annotations$lineage
names(clust_to_lineage) = as.character(annotations$cluster)
clusters[,lineage:=clust_to_lineage[as.character(cluster)]]

# Add number to index phenotypes
clust_to_phenonum = 1:15
names(clust_to_phenonum) = c("CD14_cell","CD20_cell","CD209_cell","CD31_cell","CD3_cell","CD4_cell","CD45_other","CD56_cell","CD68_cell","CD8_cell","Mast_cell","Other","PanCK_cell","SMA_cell","Vim_cell")
clusters[,phenotype_num:=clust_to_phenonum[as.character(phenotype)]]
# Save
fwrite(clusters, "seg_cell_transformed_data_clusters_annot.csv")


