# Get tissue area for each sample
# Author: Candace Liu
# Date: 2/3/21

import pandas as pd
import numpy as np
import os
import matplotlib.pyplot as plt
from PIL import Image
import scipy.ndimage as ndimage
import skimage.filters

# sigma for Gaussian blur
s = 2

# Create directory to store output
output_dir = 'tissue_area_masks'
if not os.path.exists(output_dir):
    os.makedirs(output_dir)

# all points
points = np.array(range(7,20))
paths_all_points = [os.path.join("no_noise","Point"+str(x),"TIFs") for x in points]

area_dict = dict()
for i in range(len(points)):
    point = points[i]
    path = paths_all_points[i]

    # Get carbon TIF
    carbon = np.array(Image.open(os.path.join(path,"C.tif")))
    carbon = carbon.astype(np.float64)
    
    # Blur and threshold
    carbon_blur = ndimage.gaussian_filter(carbon, sigma=s)
    t = skimage.filters.threshold_otsu(carbon_blur)
    carbon_bin = np.zeros(carbon_blur.shape)
    carbon_bin[carbon_blur > t] = 1
    area_dict[point] = np.sum(carbon_bin)

    # Save histogram and threshold
    fig,ax = plt.subplots(figsize=(20,20))
    plt.hist(carbon_blur.flatten())
    plt.axvline(t, color='black')
    fig.tight_layout()
    plt.savefig(os.path.join(output_dir,"Point"+str(point)+"_histogram.tiff"))
    plt.close()

    # Save image
    fig,ax = plt.subplots(figsize=(20,20))
    ax.imshow(carbon_bin, cmap='gray')
    fig.tight_layout()
    plt.savefig(os.path.join(output_dir,"Point"+str(point)+"_mask.tiff"))
    plt.close()


# save to data frame
area_per_pixel = (0.4**2) / (1024**2)
df = pd.DataFrame()
df['point'] = area_dict.keys()
df['area_pixels'] = area_dict.values()
df['area_mm2'] = [x*area_per_pixel for x in area_dict.values()]

df.to_csv('tissue_area.csv', index=False)

